#############################
## stateMachine.py         ##
## exemple de cours MDD    ##
## Gireg Demseulles        ##
## 19/02/2014              ##
## modif 27/07/2022        ##
############################################################
## Le module stateMachine permet de creer une machine a   ##
## etat reutilisable dans differents contextes            ##
## Il est possible d associer a chaque etat ou transition ##
## une fonction particuliere                              ##
############################################################

import time

class StateMachine : pass

#-------------
#constructeur
#-------------
def create():
	sm=StateMachine()
	sm.current=None # n
	sm.states=[] # liste de (name, function)
	sm.transitions=[] # liste de (n1,n2,function)
	return sm

#------------------
# operations states	
#------------------

def setCurrent(sm,name):
	#change l etat courant
	i=0
	for s in sm.states:
		if s[0]==name:
			break
		i=i+1
	sm.current=i

def getCurrent(sm):
	return sm.states[sm.current][0]

def addState(sm,name,function):
	state=name,function
	sm.states.append(state)

def getCurrentFunction(sm):
	#renvoie l action associee a l etat courant
	return ((sm.states[sm.current])[1])

#------------------------
# operations transitions
#------------------------
def addTransition(sm,name1,name2,funct):
	#ajoute une transition dans la machine a etat
	
	#recherche index etat 1
	i1=0
	for s in sm.states:
		if s[0]==name1:
			break
		i1=i1+1
	#recherche index etat 2
	i2=0
	for s in sm.states:
		if s[0]==name2:
			break
		i2=i2+1
		
	#ajout transition	
	t=i1,i2,funct
	sm.transitions.append(t)

def changeCurrent(sm,data=None):
	#realise la premiere transition valide
	
	
	#pour chaque transition
	for t in sm.transitions:
		#si la transition correspond a l etat courant
		if t[0]==sm.current:
			#si la fonction de transition renvoie vraie
			if (t[2](data)):
				#alors on valide la transition
				sm.current=t[1]
				return True
	return False

def display(sm):
	#affiche le contenu de la machine
	msg ="---Etat de la machine a etat---"
	print(msg)
	print(sm.current)
	print(sm.states)
	print(sm.transitions)
	
	
######################### TESTS UNITAIRES #######################	
if __name__=='__main__':
	def a(data):
		msg = "action etat a (" + data+")"
		print(msg)

	def b(data):
		print("action etat b")

	def c(data):
		print("action etat c")

	def ab(data):
		msg = "ab->false (" + data+")"
		print(msg)
		return False

	def ac(data):
		print("ac->true")
		return True

	def ca(data):
		print("ca->true")
		return True
	
	#creation machine	
	me=create()	

	#creation etats
	addState(me,"a",a)
	addState(me,"b",b)
	addState(me,"c",c)

	#selection etat courant
	setCurrent(me,"a")
	msg = "etat initial : " + getCurrent(me)	
	print(msg)
	
	#ajout des transitions
	addTransition(me,'a','b',ab)
	addTransition(me,'a','c',ac)
	addTransition(me,'c','a',ca)

	#parametre pour les fonctions d etat et de transition
	testData = "test data"

	#boucle de simulation
	i=3
	while(i):
		print("--------step--------")
		display(me)
		time.sleep(1)
		print ("--action:--")
		getCurrentFunction(me)(testData)
		time.sleep(1)
		print ("--transitions:--")
		changeCurrent(me,testData)
		i=i-1
		time.sleep(1)
		
	
		